<!DOCTYPE html>
<html>
    <head>
        <link href='/static/main.css' rel='stylesheet'></link>
        <script src='/static/jquery-3.2.1.min.js'></script>
        <script src='/static/js.cookie.min.js'></script>
        <script src='/static/showdown.min.js'></script>
        <script src='/static/Chart.min.js'></script>
        <script src='/static/main.js'></script>
    </head>
    <body>
        <nav>
            <ul class='bar'>
                <li><a>[</a></li>
                <li><a id='check_status'></a></li>
                <li><a id='score_display'>Score: -</a></li>
                <li><a>]</a></li>
                <li><a href='#' id='login'>Login</a></li>
                <li><a href='#' id='logout'>Logout</a></li>
                <li><a href='#' id='signup'>Signup</a></li>
                <li><a href='#' id='leaderboard'>Leaderboard</a></li>
            </ul>
        </nav>
            <ul id='login_form'>
                <li><input id='login_uname' placeholder='username'></input></li>
                <li><input id='login_pwd' type='password' placeholder='password'></input></li>
                <li><button id='login_button'>Login</button></li>
                <li><button id='signup_button'>Signup</button></li>
            </ul>
        <ul class='vertbar'>
            {% for qid in questions %}
            <li><a href='#' qid={{qid}} class='question_listing'>Question {{loop.index}}</a></li>
            {% endfor %}
        </ul>
        <div id='content'>
            <div id='content_question_space'>
            </div>
            <div id='submission_form'>
				<textarea id='codetext' placeholder='Your code'></textarea>
                <select id='language_selection'> </select>
                <button id='submit_code'>Submit Code</button>
            </div>
        </div>
    </body>
</html>
