#content{
    width: 85%;
    float: right;
    display: inline-grid;
    grid-template-columns: auto auto;
}
.bar {
    list-style-type: none;
    margin: 0;
    padding: 0;
    overflow: hidden;
    background-color: #333;
}
.bar li{
    float: left;
}
.bar a{
    display: block;
    color: white;
    text-align: center;
    padding: 14px 16px;
    text-decoration: none;
}
.bar a:hover {
    background-color: #111;
}
.vertbaractive {
    background-color: #4CAF50;
}
.vertbar {
    list-style-type: none;
    margin: 0;
    padding: 0;
    width: 13%;
    background-color: #f1f1f1;
    height: 100%; /* Full height */
    position: fixed; /* Make it stick, even on scroll */
    overflow: auto; /* Enable scrolling if the sidenav has too much content */
 }
.vertbar a {
    display: block;
    color: #000;
    padding: 8px 16px;
    text-decoration: none;
}

.vertbar a.active {
    background-color: #4CAF50;
    color: white;
}

.vertbar a:hover:not(.active) {
    background-color: #555;
    color: white;
}
#login_form, #submission_form{
    list-style-type: none;
    border-left: 3px solid pink;
    padding-left: 4px;
}
#login_form input, #submission_form textarea{
    border: 0;
    box-shadow: none;
    border-radius: 0px;
    background: lightgray;
    color: black;
}
#submission_form textarea{
    height: 400px;
}
#login_form button, #submission_form button{
    border: 1px solid;
    box-shadow: none;
    border-radius: 0px;
    color: black;
}
#submission_form {
    display: inline-grid;
    grid-template-rows: auto auto;
    grid-row-gap: 5px;
}
